#ifndef STROPS_H
#define STROPS_H

#include <stddef.h>

/*
    Functions that return char*, return a new heap-allocated string.
    The user of this library is responsible for freeing the memory of the result.
    Only 7-Bit Ascii is supported.
*/

typedef unsigned long long ull_t;
typedef unsigned char bool_t;

ull_t strops_length(const char* string);
char* strops_copy(const char* string);
char* strops_copy_amount(const char* string, ull_t amount);

ull_t strops_first_pos_of_char(const char* string, char char_to_search);
ull_t strops_first_pos_of_string(const char* string, const char* string_to_search);

bool_t strops_contains_char(const char* string, char char_to_search);
bool_t strops_contains_string(const char* string, const char* string_to_search);
bool_t strops_equals(const char* string1, const char* string2);
bool_t strops_starts_with(const char* string1, const char* string2);

char* strops_to_lowercase(const char* string);
char* strops_to_uppercase(const char* string);
bool_t strops_is_lowercase(const char* string);
bool_t strops_is_uppercase(const char* string);

char* strops_insert_at_pos_string(const char* string, const char* string_to_insert, ull_t pos);
char* strops_remove_at_pos_char(const char* string, ull_t pos);
void strops_remove_at_pos_char_inplace(char* string, ull_t pos);

char* strops_trim_right_whitespace(const char* string);
char* strops_trim_left_whitespace(const char* string);
char* strops_trim_both_whitespace(const char* string);

char* strops_trim_right_chars(const char* string, const char* chars_to_remove);
char* strops_trim_left_chars(const char* string, const char* chars_to_remove);
char* strops_trim_both_chars(const char* string, const char* chars_to_remove);

char* strops_trim_right_string(const char* string, const char* string_to_remove);
char* strops_trim_left_string(const char* string, const char* string_to_remove);
char* strops_trim_both_string(const char* string, const char* string_to_remove);

char* strops_remove_chars(const char* string, const char* chars_to_remove);
char* strops_remove_string(const char* string, const char* string_to_remove);

ull_t strops_word_count(const char* string);

#endif /* STROPS_H */

